const os = require('os');
const net = require('net');

/**
 * @description: 获取当前机器的ip地址
 * @return {*}
 */
function getIpAddress() {
  let ifaces = os.networkInterfaces();

  for (let dev in ifaces) {
    let iface = ifaces[dev];

    for (let i = 0; i < iface.length; i++) {
      let { family, address, internal } = iface[i];

      if (family === 'IPv4' && address !== '127.0.0.1' && !internal) {
        return address;
      }
    }
  }
}

/**
 * @description: 检查端口是否被占用
 * @param {*} port
 * @return {*}
 */
function checkPortIsFree(port) {
  return new Promise((resolve, reject) => {
    let server = net.createServer().listen(port);
    server.on('listening', function () {
      server.close();
      resolve(port);
    });
    server.on('error', function (err) {
      if (err.code === 'EADDRINUSE') {
        reject(err);
      }
    });
  });
}

async function getFreePort(port = 8000) {
  while (true) {
    try {
      return await checkPortIsFree(port);
    } catch (err) {
      return await getFreePort(++port);
    }
  }
}

module.exports = {
  getIpAddress,
  checkPortIsFree,
  getFreePort,
};
