package handler

import (
	"context"
	"encoding/base64"
	"encoding/json"

	"code.byted.org/developer/open_microapp_passport_showcase/go/conf"
	"code.byted.org/developer/open_microapp_passport_showcase/go/model"
	"code.byted.org/developer/open_microapp_passport_showcase/go/store"
	"code.byted.org/developer/open_microapp_passport_showcase/go/utils"
)

const (
	domain           = "developer.toutiao.com"
	sandBoxDomain    = "open-sandbox.douyin.com"
	code2SessionPath = "/api/apps/v2/jscode2session"
)

type code2SessionRequest struct {
	AppId  string `json:"appid"`
	Secret string `json:"secret"`
	Code   string `json:"code"`
	ACode  string `json:"anonymous_code"`
}

type Code2SessionResponseData struct {
	SessionKey string `json:"session_key"`
	Openid     string `json:"openid"`
	AOpenId    string `json:"anonymous_openid"`
	UnionId    string `json:"unionid"`
	DOpenid    string `json:"dopenid"`
}

type Code2SessionResponse struct {
	ErrNo   int64  `json:"err_no"`
	ErrTips string `json:"err_tips"`

	Data Code2SessionResponseData `json:"data"`
}

func code2Session(ctx context.Context, req *code2SessionRequest) (*Code2SessionResponse, error) {
	reqBodyByte, _ := json.Marshal(req)
	reqDomain := domain
	if conf.IsSandBox == "1" {
		reqDomain = sandBoxDomain
	}
	respBody, err := utils.HttpDo(ctx, code2SessionPath, utils.HttpPostMethod, string(reqBodyByte), "https", reqDomain)
	resp := &Code2SessionResponse{}
	if err != nil {
		return nil, err
	}

	err = json.Unmarshal([]byte(respBody), resp)

	return resp, err
}

func ShowData(context context.Context, req *model.ShowDataRequest) *model.ShowDataResponse {
	userData, exist := store.GetCache(context, req.Token)
	if !exist {
		resp := &model.ShowDataResponse{
			ErrCode:       -1,
			ErrMsg:        "",
			DecryptedData: "",
			Token:         "",
		}
		return resp
	}

	// 上线前依据业务日志需要自行删除
	println("data: ", req.EncryptedData.Data)
	println("sessionkey: ", userData.SessionKey)
	println("iv: ", req.EncryptedData.Iv)
	decodeData, _ := base64.StdEncoding.DecodeString(req.EncryptedData.Data)
	decodeSecret, _ := base64.StdEncoding.DecodeString(userData.SessionKey)
	decodeIv, _ := base64.StdEncoding.DecodeString(req.EncryptedData.Iv)
	decryptData, err := utils.CbcDecrypt(decodeData, decodeSecret, decodeIv)
	if err != nil {
		resp := &model.ShowDataResponse{
			ErrCode:       -1,
			ErrMsg:        err.Error(),
			DecryptedData: "",
			Token:         "",
		}
		return resp
	}
	token := store.SetCache(context, req.Token, userData.OpenId, userData.UnionId, decryptData, userData.SessionKey)

	resp := &model.ShowDataResponse{
		ErrCode:       0,
		ErrMsg:        "success",
		DecryptedData: utils.MaskPhoneNumber(context, decryptData),
		Token:         token,
	}
	return resp
}
