package main

import (
	"context"
	"encoding/json"
	"os"
	"strings"

	"code.byted.org/developer/open_microapp_passport_showcase/go/conf"
	"code.byted.org/developer/open_microapp_passport_showcase/go/handler"
	"code.byted.org/developer/open_microapp_passport_showcase/go/model"

	"github.com/cloudwego/hertz/pkg/app"
	"github.com/cloudwego/hertz/pkg/app/server"
	"github.com/cloudwego/hertz/pkg/protocol/consts"
)

func main() {
	args := os.Args
	for _, v := range args {
		println(v)
		strings.Split(v, "=")
	}

	conf.ParseArgs(args)

	h := server.New(server.WithHostPorts(conf.HostPorts))

	h.POST("/api/apps/showData", func(c context.Context, ctx *app.RequestContext) {
		req := &model.ShowDataRequest{}
		reqBody, _ := ctx.Body()
		_ = json.Unmarshal(reqBody, req)
		resp := handler.ShowData(c, req)
		ctx.JSON(consts.StatusOK, resp)
	})
	h.POST("/api/apps/login", func(c context.Context, ctx *app.RequestContext) {
		req := &model.LoginRequest{}
		reqBody, _ := ctx.Body()
		_ = json.Unmarshal(reqBody, req)
		resp := handler.Login(c, req)
		ctx.JSON(consts.StatusOK, resp)
	})
	h.POST("/api/apps/userData", func(c context.Context, ctx *app.RequestContext) {
		req := &model.UserDataRequest{}
		reqBody, _ := ctx.Body()
		_ = json.Unmarshal(reqBody, req)
		resp := handler.UserData(c, req)
		ctx.JSON(consts.StatusOK, resp)
	})
	h.POST("/api/apps/logout", func(c context.Context, ctx *app.RequestContext) {
		req := &model.LogoutRequest{}
		reqBody, _ := ctx.Body()
		_ = json.Unmarshal(reqBody, req)
		resp := handler.Logout(c, req)
		ctx.JSON(consts.StatusOK, resp)
	})

	h.Spin()
}
