package open.douyin.com.handler;

import com.google.protobuf.util.JsonFormat;
import open.douyin.com.conf.AppConfig;
import open.douyin.com.model.Code2SessionRequest;
import open.douyin.com.model.Code2SessionResponse;
import open.douyin.com.store.Store;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import com.google.gson.FieldNamingPolicy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import org.springframework.web.bind.annotation.RestController;
import open.douyin.com.utils.HttpUtil;

import java.io.IOException;

@RestController
@ComponentScan
public class Login {

    @Autowired
    private AppConfig appConfig;

    /**
     * 小程序利用code2session登录获取openid返回
     * @param requestBody
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/api/apps/login", consumes = MediaType.APPLICATION_JSON_VALUE)
    String handle(@RequestBody String requestBody) throws IOException {
        // 请求体初始化pb结构
        open.douyin.com.model.Login.LoginRequest.Builder requestBuild = open.douyin.com.model.Login.LoginRequest.newBuilder();
        JsonFormat.parser().merge(requestBody, requestBuild);
        open.douyin.com.model.Login.LoginRequest request = requestBuild.build();

        // 装配code2session请求体
        Gson gson = new GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.UPPER_CAMEL_CASE).create();
        Code2SessionRequest code2SessionRequest = new Code2SessionRequest();
        code2SessionRequest.setCode(request.getCode());
        code2SessionRequest.setAppid(appConfig.AppId);
        code2SessionRequest.setSecret(appConfig.Secret);

        // code2session的http请求 TODO 常量化
        String domain = "developer.toutiao.com";
        if (appConfig.IsSandBox.equals("1")) {
            domain = "open-sandbox.douyin.com";
        }
        String code2SessionPath = "/api/apps/v2/jscode2session";
        Code2SessionResponse response = HttpUtil.HttpPost(code2SessionPath, gson.toJson(code2SessionRequest), "https", domain, Code2SessionResponse.class);

        // 初始化登录返回数据 并处理返回异常
        open.douyin.com.model.Login.LoginResponse.Builder builder = open.douyin.com.model.Login.LoginResponse.newBuilder();

        // 对返回数据异常处理
        if (response == null || response.getData() == null) {
            builder.setErrCode(-1);
            builder.setErrMsg("error");
            open.douyin.com.model.Login.LoginResponse obj = builder.build();
            return JsonFormat.printer().print(obj);
        }
        if (response.getErrNo() != 0) {
            builder.setErrCode(-1);
            builder.setErrMsg(response.getErrTips());
            open.douyin.com.model.Login.LoginResponse obj = builder.build();
            return JsonFormat.printer().print(obj);
        }

        // 返回成功则组装返回结构体
        builder.setErrCode(response.getErrNo());
        builder.setErrMsg("");
        builder.setErrCode(0);

        String token = Store.SetCache("", response.getData().getOpenid(), response.getData().getUnionId(), "", response.getData().getSessionKey());
        builder.setToken(token);

        builder.setOpenid(response.getData().getOpenid().substring(0, 3) + "****" + response.getData().getOpenid().substring(7));
        builder.setUnionid(response.getData().getUnionId().substring(0, 3) + "****" + response.getData().getUnionId().substring(7));

        open.douyin.com.model.Login.LoginResponse responseBody = builder.build();
        return JsonFormat.printer().print(responseBody);
    }

}