package open.douyin.com.handler;

import com.google.protobuf.util.JsonFormat;
import open.douyin.com.conf.AppConfig;
import open.douyin.com.model.Login;
import open.douyin.com.store.Store;
import open.douyin.com.utils.CbcCoding;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import open.douyin.com.store.Mask;
import open.douyin.com.store.UserSession;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

@RestController
public class ShowData {

    @Autowired
    private AppConfig appConfig;

    /**
     * 小程序登录解密全流程，code2session->cbc decryption，这里demo对可能存在的加解密异常不做额外处理，开发者可以自行加入处理逻辑
     * @param requestBody
     * @return
     * @throws IOException
     * @throws InvalidAlgorithmParameterException
     * @throws NoSuchPaddingException
     * @throws IllegalBlockSizeException
     * @throws NoSuchAlgorithmException
     * @throws BadPaddingException
     * @throws InvalidKeyException
     */
    @RequestMapping(value = "/api/apps/showData", consumes = MediaType.APPLICATION_JSON_VALUE)
    public String handle(@RequestBody String requestBody) throws IOException, InvalidAlgorithmParameterException, NoSuchPaddingException, IllegalBlockSizeException, NoSuchAlgorithmException, BadPaddingException, InvalidKeyException {
        // 装载showdata请求体
        open.douyin.com.model.Login.ShowDataRequest.Builder requestBuild = open.douyin.com.model.Login.ShowDataRequest.newBuilder();
        JsonFormat.parser().merge(requestBody, requestBuild);
        open.douyin.com.model.Login.ShowDataRequest request = requestBuild.build();

        UserSession userData = Store.GetCache(request.getToken());

        open.douyin.com.model.Login.ShowDataResponse.Builder builder = open.douyin.com.model.Login.ShowDataResponse.newBuilder();

        // cbc解密
        open.douyin.com.model.Login.EncryptedData data = request.getEncryptedData();
        String decryptData = CbcCoding.decrypt(data.getData(), userData.getSessionKey(), data.getIv());
        builder.setDecryptedData(Mask.MaskPhoneNumber(decryptData));
        String token = Store.SetCache(request.getToken(), userData.getOpenid(), userData.getUnionid(), decryptData, userData.getSessionKey());

        builder.setToken(token);
        Login.ShowDataResponse obj = builder.build();

        return JsonFormat.printer().print(obj);
    }
}
